namespace Tarpit;

public class WordPredictor
{
    private Dictionary<string, List<string>> _wordFrequencies = new();
    
    public WordPredictor(string filePath)
    {
        LoadData(filePath);
    }
    
    private void LoadData(string filePath)
    {
        _wordFrequencies = new();
        
        try
        {
            using StreamReader reader = new(filePath);
            string? previousWord = null;
            
            while (reader.ReadLine() is { } line)
            {
                string[] words = line.Split(new char[] { ' ', '.', ',', '!', '?', ';', ':', '(', ')', '\n', '\r', '\t' }, StringSplitOptions.RemoveEmptyEntries);
                
                foreach (string word in words)
                {
                    if (previousWord != null)
                    {
                        if (!_wordFrequencies.ContainsKey(previousWord))
                        {
                            _wordFrequencies[previousWord] = [];
                        }
                        _wordFrequencies[previousWord].Add(word);
                    }
                    previousWord = word;
                }
            }
        }
        catch (FileNotFoundException)
        {
            Console.WriteLine($"Error: File not found at {filePath}");
            Environment.Exit(1); // Exit the program if the file isn't found.  Important!
        }
        catch (Exception ex)
        {
            Console.WriteLine($"An error occurred while reading the file: {ex.Message}");
            Environment.Exit(1);
        }
    }
    
    public List<string?> PredictNextWord(string inputWord)
    {
        if (!_wordFrequencies.ContainsKey(inputWord.ToLower()))
        {
            return ["Word not found in training data."];
        }
        
        List<string> nextWords = _wordFrequencies[inputWord.ToLower()];
        
        List<string?> words = [];
        
        for (int i = 0; i < 10; i++)
        {
            string? mostFrequentWord = nextWords
                .GroupBy(w => w)
                .OrderByDescending(g => g.Count())
                .ElementAtOrDefault(i)
                ?.Key;
            
            words.Add(mostFrequentWord);
        }
        
        return words;
    }
}